"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.nodeMergeAddIds = exports.nodeMergeLanguageTerm = exports.nodeMergeOverwrite = exports.addEmptyArray = exports.addInheritanceTermsDataTypesAndProperties = exports.addInheritanceTermsClassAndEnum = exports.extractFromClassMemory = exports.getStandardContext = exports.checkIfNamespaceFromListIsUsed = exports.discoverEquateNamespaces = exports.discoverUsedSchemaOrgProtocol = exports.preProcessVocab = exports.generateContext = exports.curateVocabNode = void 0;
const namespaces_1 = require("./data/namespaces");
const jsonld_1 = __importDefault(require("jsonld"));
const cloneJson_1 = require("./utilities/cloneJson");
const isObject_1 = require("./utilities/isObject");
const isString_1 = require("./utilities/isString");
const isNil_1 = require("./utilities/isNil");
const isLanguageObjectVocab_1 = require("./utilities/isLanguageObjectVocab");
const isArray_1 = require("./utilities/isArray");
const switchIRIProtocol_1 = require("./utilities/switchIRIProtocol");
function curateLanguageTerm(vocabNode, term) {
    if (vocabNode[term] !== undefined) {
        if ((0, isString_1.isString)(vocabNode[term])) {
            vocabNode[term] = {
                en: vocabNode[term],
            };
        }
        else if ((0, isLanguageObjectVocab_1.isLanguageObjectVocab)(vocabNode[term])) {
            vocabNode[term] = {
                [vocabNode[term]["@language"]]: vocabNode[term]["@value"],
            };
        }
        else if ((0, isArray_1.isArray)(vocabNode[term])) {
            const newVal = {};
            vocabNode[term].map((el) => {
                if ((0, isLanguageObjectVocab_1.isLanguageObjectVocab)(el)) {
                    newVal[el["@language"]] = el["@value"];
                }
            });
            vocabNode[term] = newVal;
        }
    }
    else {
        vocabNode[term] = {};
    }
}
function curateRelationshipTermArray(vocabNode, term, initDefaultIf) {
    if ((0, isString_1.isString)(vocabNode[term])) {
        vocabNode[term] = [vocabNode[term]];
    }
    else if (vocabNode[term] === undefined &&
        vocabNode["@type"] === initDefaultIf) {
        vocabNode[term] = [];
    }
}
function curateVocabNode(vocabNode, vocabularies) {
    curateLanguageTerm(vocabNode, namespaces_1.NS.rdfs.comment);
    curateLanguageTerm(vocabNode, namespaces_1.NS.rdfs.label);
    curateRelationshipTermArray(vocabNode, namespaces_1.NS.rdfs.subClassOf, namespaces_1.TermTypeIRI.class);
    curateRelationshipTermArray(vocabNode, namespaces_1.NS.rdfs.subPropertyOf, namespaces_1.TermTypeIRI.property);
    curateRelationshipTermArray(vocabNode, namespaces_1.NS.schema.domainIncludes, namespaces_1.TermTypeIRI.property);
    curateRelationshipTermArray(vocabNode, namespaces_1.NS.schema.rangeIncludes, namespaces_1.TermTypeIRI.property);
    if (vocabNode[namespaces_1.NS.schema.inverseOf] === undefined &&
        vocabNode["@type"] === namespaces_1.TermTypeIRI.property) {
        vocabNode[namespaces_1.NS.schema.inverseOf] = null;
    }
    if (!(0, isString_1.isString)(vocabNode[namespaces_1.NS.schema.isPartOf])) {
        const vocabKeys = Object.keys(vocabularies);
        let vocab = vocabKeys.find((el) => vocabNode["@id"].substring(0, vocabNode["@id"].indexOf(":")) === el);
        if ((0, isString_1.isString)(vocab)) {
            vocab = vocabularies[vocab];
            let newChange;
            do {
                newChange = false;
                if (vocab.endsWith("/") || vocab.endsWith("#")) {
                    vocab = vocab.substring(0, vocab.length - 1);
                    newChange = true;
                }
            } while (newChange);
            vocabNode[namespaces_1.NS.schema.isPartOf] = vocab;
        }
    }
    return vocabNode;
}
exports.curateVocabNode = curateVocabNode;
function generateContext(currentContext, newContext) {
    const keysCurrentContext = Object.keys(currentContext);
    const keysNewContext = Object.keys(newContext);
    let resultContext = (0, cloneJson_1.cloneJson)(currentContext);
    for (const keyNC of keysNewContext) {
        if ((0, isString_1.isString)(newContext[keyNC])) {
            let foundMatch = false;
            for (const keyCC of keysCurrentContext) {
                if ((0, isString_1.isString)(resultContext[keyCC])) {
                    if (resultContext[keyCC] === newContext[keyNC]) {
                        foundMatch = true;
                        break;
                    }
                }
            }
            if (foundMatch) {
                continue;
            }
            if (!resultContext[keyNC]) {
                resultContext[keyNC] = newContext[keyNC];
            }
            else {
                if (resultContext[keyNC] !== newContext[keyNC]) {
                    let foundFreeName = false;
                    let counter = 1;
                    while (!foundFreeName) {
                        const newVocabIndicator = keyNC + counter++;
                        if (!resultContext[newVocabIndicator]) {
                            foundFreeName = true;
                            resultContext[newVocabIndicator] = newContext[keyNC];
                        }
                    }
                }
            }
        }
    }
    const ordered = {};
    Object.keys(resultContext)
        .sort()
        .forEach(function (key) {
        ordered[key] = resultContext[key];
    });
    resultContext = ordered;
    const keysResultContext = Object.keys(resultContext);
    const orderedResultContext = {};
    for (const keyRC of keysResultContext) {
        if ((0, isString_1.isString)(resultContext[keyRC])) {
            orderedResultContext[keyRC] = resultContext[keyRC];
        }
    }
    for (const keyRC of keysResultContext) {
        if ((0, isObject_1.isObject)(resultContext[keyRC])) {
            orderedResultContext[keyRC] = resultContext[keyRC];
        }
    }
    return orderedResultContext;
}
exports.generateContext = generateContext;
async function preProcessVocab(vocab, newContext) {
    let foundInnerGraph = false;
    do {
        const newGraph = [];
        foundInnerGraph = false;
        for (let i = 0; i < vocab["@graph"].length; i++) {
            if (vocab["@graph"][i]["@graph"] !== undefined) {
                newGraph.push(...(0, cloneJson_1.cloneJson)(vocab["@graph"][i]["@graph"]));
                foundInnerGraph = true;
            }
            else {
                newGraph.push((0, cloneJson_1.cloneJson)(vocab["@graph"][i]));
            }
        }
        vocab["@graph"] = (0, cloneJson_1.cloneJson)(newGraph);
    } while (foundInnerGraph);
    const compactedVocab = await jsonld_1.default.compact(vocab, newContext);
    if (compactedVocab["@graph"] === undefined) {
        delete compactedVocab["@context"];
        return {
            "@context": newContext,
            "@graph": [compactedVocab],
        };
    }
    else {
        return compactedVocab;
    }
}
exports.preProcessVocab = preProcessVocab;
function discoverUsedSchemaOrgProtocol(vocabulary) {
    const httpsIRI = "https://schema.org/";
    const httpIRI = "http://schema.org/";
    if (vocabulary["@context"]) {
        for (const contextEntry of Object.values(vocabulary["@context"])) {
            if ((0, isObject_1.isObject)(contextEntry) && contextEntry["@vocab"]) {
                if (contextEntry["@vocab"] === httpsIRI) {
                    return "https";
                }
                else if (contextEntry["@vocab"] === httpIRI) {
                    return "http";
                }
            }
            else if ((0, isString_1.isString)(contextEntry)) {
                if (contextEntry === httpsIRI) {
                    return "https";
                }
                else if (contextEntry === httpIRI) {
                    return "http";
                }
            }
        }
    }
    const stringifiedVocab = JSON.stringify(vocabulary);
    const amountHttps = stringifiedVocab.split(httpsIRI).length - 1;
    const amountHttp = stringifiedVocab.split(httpIRI).length - 1;
    if (amountHttps > amountHttp) {
        return "https";
    }
    else if (amountHttp > amountHttps) {
        return "http";
    }
    else {
        return httpsIRI;
    }
}
exports.discoverUsedSchemaOrgProtocol = discoverUsedSchemaOrgProtocol;
function discoverEquateNamespaces(currentContext, vocabulary) {
    const result = new Set();
    const protocolSwitchedNamespaces = [];
    Object.values(currentContext).forEach(function (el) {
        if ((0, isString_1.isString)(el)) {
            protocolSwitchedNamespaces.push((0, switchIRIProtocol_1.switchIRIProtocol)(el));
        }
    });
    if (vocabulary["@context"]) {
        Object.values(vocabulary["@context"]).forEach(function (el) {
            if ((0, isString_1.isString)(el) && protocolSwitchedNamespaces.includes(el)) {
                result.add(el);
            }
        });
    }
    if (Array.isArray(vocabulary["@graph"])) {
        vocabulary["@graph"].forEach(function (vocabNode) {
            checkIfNamespaceFromListIsUsed(vocabNode["@id"], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode["@type"], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode[namespaces_1.NS.rdfs.subClassOf], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode["http://www.w3.org/2000/01/rdf-schema#subClassOf"], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode[namespaces_1.NS.schema.domainIncludes], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode["http://schema.org/domainIncludes"], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode["https://schema.org/domainIncludes"], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode[namespaces_1.NS.schema.rangeIncludes], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode["http://schema.org/rangeIncludes"], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode["https://schema.org/rangeIncludes"], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode[namespaces_1.NS.rdfs.subPropertyOf], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode["http://www.w3.org/2000/01/rdf-schema#subPropertyOf"], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode[namespaces_1.NS.schema.inverseOf], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode["http://schema.org/inverseOf"], protocolSwitchedNamespaces, result);
            checkIfNamespaceFromListIsUsed(vocabNode["https://schema.org/inverseOf"], protocolSwitchedNamespaces, result);
        });
    }
    return Array.from(result);
}
exports.discoverEquateNamespaces = discoverEquateNamespaces;
function checkIfNamespaceFromListIsUsed(value, namespaceArray, result) {
    if (Array.isArray(value)) {
        value.forEach(function (val) {
            checkIfNamespaceFromListIsUsed(val, namespaceArray, result);
        });
    }
    else {
        let toCheck;
        if ((0, isObject_1.isObject)(value) && (0, isString_1.isString)(value["@id"])) {
            toCheck = value["@id"];
        }
        else {
            toCheck = value;
        }
        if ((0, isString_1.isString)(toCheck) && toCheck.startsWith("http")) {
            const match = namespaceArray.find((el) => toCheck.startsWith(el));
            if (match && !result.has(match)) {
                result.add(match);
            }
        }
    }
}
exports.checkIfNamespaceFromListIsUsed = checkIfNamespaceFromListIsUsed;
function getStandardContext() {
    const standardContext = {
        rdf: namespaces_1.NS.rdf._url,
        rdfs: namespaces_1.NS.rdfs._url,
        xsd: namespaces_1.NS.xsd._url,
        dcterms: namespaces_1.NS.dcterms._url,
        soa: namespaces_1.NS.soa._url,
        ds: namespaces_1.NS.ds._url,
    };
    const idEntries = [
        namespaces_1.NS.soa.superClassOf,
        namespaces_1.NS.soa.superPropertyOf,
        namespaces_1.NS.soa.hasProperty,
        namespaces_1.NS.soa.isRangeOf,
        namespaces_1.NS.soa.hasEnumerationMember,
        namespaces_1.NS.soa.enumerationDomainIncludes,
        namespaces_1.NS.rdfs.subClassOf,
        namespaces_1.NS.rdfs.subPropertyOf,
        namespaces_1.NS.schema.isPartOf,
        namespaces_1.NS.schema.domainIncludes,
        namespaces_1.NS.schema.rangeIncludes,
        namespaces_1.NS.schema.supersededBy,
        namespaces_1.NS.schema.inverseOf,
        namespaces_1.NS.schema.source,
        namespaces_1.NS.dcterms.source,
    ];
    idEntries.map((el) => {
        standardContext[el] = {
            "@id": el,
            "@type": "@id",
        };
    });
    return standardContext;
}
exports.getStandardContext = getStandardContext;
function extractFromClassMemory(classMemory, otherMemory, addGraphNodeFn, vocabURL) {
    let termSwitched;
    do {
        termSwitched = false;
        const classesKeys = Object.keys(classMemory);
        const otherKeys = Object.keys(otherMemory);
        for (const actClassKey of classesKeys) {
            if (otherKeys.includes(actClassKey)) {
                termSwitched = true;
                addGraphNodeFn(otherMemory, classMemory[actClassKey], vocabURL);
                delete classMemory[actClassKey];
            }
            else if (classMemory[actClassKey][namespaces_1.NS.rdfs.subClassOf] !== undefined) {
                const subClassArray = classMemory[actClassKey][namespaces_1.NS.rdfs.subClassOf];
                for (const actSubClass of subClassArray) {
                    if (actSubClass === namespaces_1.TermTypeIRI.enumeration ||
                        otherKeys.includes(actSubClass)) {
                        if (classMemory[actClassKey] && !otherMemory[actClassKey]) {
                            termSwitched = true;
                            otherMemory[actClassKey] = (0, cloneJson_1.cloneJson)(classMemory[actClassKey]);
                            delete classMemory[actClassKey];
                        }
                        else if (classMemory[actClassKey] && otherMemory[actClassKey]) {
                            termSwitched = true;
                            addGraphNodeFn(otherMemory, classMemory[actClassKey], vocabURL);
                            delete classMemory[actClassKey];
                        }
                    }
                }
            }
        }
    } while (termSwitched);
}
exports.extractFromClassMemory = extractFromClassMemory;
function addInheritanceTermsClassAndEnum(memory, enumerationsMemory, subOfProperty, superOfProperty) {
    const classesKeys = Object.keys(memory);
    for (const actClassKey of classesKeys) {
        const superClasses = memory[actClassKey][subOfProperty];
        if (!memory[actClassKey][superOfProperty]) {
            memory[actClassKey][superOfProperty] = [];
        }
        for (const actSuperClass of superClasses) {
            let superClass = memory[actSuperClass];
            if (!superClass) {
                superClass = enumerationsMemory[actSuperClass];
            }
            if (superClass) {
                if (superClass[superOfProperty]) {
                    if (!superClass[superOfProperty].includes(actClassKey)) {
                        superClass[superOfProperty].push(actClassKey);
                    }
                }
                else {
                    superClass[superOfProperty] = [actClassKey];
                }
            }
        }
    }
}
exports.addInheritanceTermsClassAndEnum = addInheritanceTermsClassAndEnum;
function addInheritanceTermsDataTypesAndProperties(memory, subOfProperty, superOfProperty) {
    const dataTypeKeys = Object.keys(memory);
    for (const actDtKey of dataTypeKeys) {
        const superClasses = memory[actDtKey][subOfProperty];
        if (!memory[actDtKey][superOfProperty]) {
            memory[actDtKey][superOfProperty] = [];
        }
        if (!superClasses) {
            memory[actDtKey][subOfProperty] = [];
        }
        else {
            for (const actSuperClass of superClasses) {
                const superClass = memory[actSuperClass];
                if (superClass) {
                    if (superClass[superOfProperty]) {
                        if (!superClass[superOfProperty].includes(actDtKey)) {
                            superClass[superOfProperty].push(actDtKey);
                        }
                    }
                    else {
                        superClass[superOfProperty] = [actDtKey];
                    }
                }
            }
        }
    }
}
exports.addInheritanceTermsDataTypesAndProperties = addInheritanceTermsDataTypesAndProperties;
function addEmptyArray(termObject, property) {
    if (!termObject[property]) {
        termObject[property] = [];
    }
}
exports.addEmptyArray = addEmptyArray;
function nodeMergeOverwrite(oldNode, newNode, property) {
    if (!(0, isNil_1.isNil)(newNode[property])) {
        oldNode[property] = newNode[property];
    }
}
exports.nodeMergeOverwrite = nodeMergeOverwrite;
function nodeMergeLanguageTerm(oldNode, newNode, property) {
    if (!(0, isNil_1.isNil)(newNode[property])) {
        const langKeys = Object.keys(newNode[property]);
        for (const actLangKey of langKeys) {
            oldNode[property][actLangKey] = newNode[property][actLangKey];
        }
    }
}
exports.nodeMergeLanguageTerm = nodeMergeLanguageTerm;
function nodeMergeAddIds(oldNode, newNode, property) {
    if (!(0, isNil_1.isNil)(newNode[property])) {
        for (const arrayElement of newNode[property]) {
            if (!oldNode[property].includes(arrayElement)) {
                oldNode[property].push(arrayElement);
            }
        }
    }
}
exports.nodeMergeAddIds = nodeMergeAddIds;
//# sourceMappingURL=graphUtilities.js.map